local MakePlayerCharacter = require "prefabs/player_common"

local assets = {
        Asset( "ANIM", "anim/player_basic.zip" ),
        Asset( "ANIM", "anim/player_idles_shiver.zip" ),
        Asset( "ANIM", "anim/player_actions.zip" ),
        Asset( "ANIM", "anim/player_actions_axe.zip" ),
        Asset( "ANIM", "anim/player_actions_pickaxe.zip" ),
        Asset( "ANIM", "anim/player_actions_shovel.zip" ),
        Asset( "ANIM", "anim/player_actions_blowdart.zip" ),
        Asset( "ANIM", "anim/player_actions_eat.zip" ),
        Asset( "ANIM", "anim/player_actions_item.zip" ),
        Asset( "ANIM", "anim/player_actions_uniqueitem.zip" ),
        Asset( "ANIM", "anim/player_actions_bugnet.zip" ),
        Asset( "ANIM", "anim/player_actions_fishing.zip" ),
        Asset( "ANIM", "anim/player_actions_boomerang.zip" ),
        Asset( "ANIM", "anim/player_bush_hat.zip" ),
        Asset( "ANIM", "anim/player_attacks.zip" ),
        Asset( "ANIM", "anim/player_idles.zip" ),
        Asset( "ANIM", "anim/player_rebirth.zip" ),
        Asset( "ANIM", "anim/player_jump.zip" ),
        Asset( "ANIM", "anim/player_amulet_resurrect.zip" ),
        Asset( "ANIM", "anim/player_teleport.zip" ),
        Asset( "ANIM", "anim/wilson_fx.zip" ),
        Asset( "ANIM", "anim/player_one_man_band.zip" ),
        Asset( "ANIM", "anim/shadow_hands.zip" ),
        Asset( "SOUND", "sound/sfx.fsb" ),
        Asset( "SOUND", "sound/wilson.fsb" ),
        Asset( "ANIM", "anim/beard.zip" ),
        Asset( "ANIM", "anim/dementor.zip" ),
        Asset( "ANIM", "anim/flowerc.zip" ),
}

local prefabs = {"flower_corrupt"}
local start_inv = {}

local dementorMaxSanity = 100
local dementorMaxHealth = 120
local dementorAuraMaxRange = 20

local auraRange = 3

function GetSpecialCharacterString(character)
    character = string.lower(character)
    if character == "dementor" then
		return "..."
	elseif character == "wes" then
		return ""
    --else
		--print (character)
    end
end	

local function switchAuraDisplay(inst)
    if GetClock():IsNight() then
		inst.Light:Enable(false)
	else
		inst.Light:Enable(true)
    end
end

-- Bunny turning functions, copied from their prefabs since they're not available to call otherwise --
local function SetBeardlord(inst)
	if not inst.beardlord then
		inst.AnimState:SetBuild("manrabbit_beard_build")    
		inst.beardlord = true
        inst.components.combat:SetDefaultDamage(TUNING.BEARDLORD_DAMAGE)
        inst.components.combat:SetAttackPeriod(TUNING.BEARDLORD_ATTACK_PERIOD)
        inst.components.combat.panic_thresh = TUNING.BEARDLORD_PANIC_THRESH
        inst.components.lootdropper:SetLoot({"beardhair", "beardhair", "monstermeat"})
        inst.components.sleeper:SetSleepTest(function() return false end)
        inst.components.sleeper:SetWakeTest(function() return true end)
	end
end

local beardsounds = 
{
    scream = "dontstarve/rabbit/beardscream",
    hurt = "dontstarve/rabbit/beardscream_short",
}
local function BecomeBeardling(inst)
	if inst.israbbit or inst.iswinterrabbit then
		inst.AnimState:SetBuild("beard_monster")
	    inst.components.lootdropper:SetLoot{}
		inst.components.lootdropper:AddRandomLoot("beardhair", .5)	    
		inst.components.lootdropper:AddRandomLoot("monstermeat", 1)	    
		inst.components.lootdropper:AddRandomLoot("nightmarefuel", 1)	  
		inst.components.lootdropper.numrandomloot = 1  
		inst.components.sanityaura.aura = -TUNING.SANITYAURA_MED		
	    inst.israbbit = false
	    inst.iswinterrabbit = false
	    inst.components.inventoryitem:ChangeImageName("beard_monster")
		inst.sounds = beardsounds
	end
end

local function onpickedfn(inst, picker)
	if picker and picker.components.sanity then
		picker.components.sanity:DoDelta(TUNING.SANITY_TINY)
	end	
	inst:Remove()
end

local function BecomeCorruptFlower(inst)
	-- Thanks to reinert from the Don't Starve Mod Forums for this code snippet --
	local log = SpawnPrefab('flower_corrupt')
	log.AnimState:PlayAnimation(inst.animname)
	log.Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst:Remove()
    print "flower corrupted"
end

local function validateTarget(target)
	if (target:HasTag("shadowcreature") or target:HasTag("chess") or target:HasTag("ghost") or target:HasTag("wall") or target:HasTag("structure") or target.prefab == "chester") then
		return false
	-- If a rabbit is affected, turn him bearded at mid-life --
	elseif (target:HasTag("rabbit") and target.components.health.currenthealth <= TUNING.RABBIT_HEALTH / 2) then
		if target.checktask then target.checktask:Cancel() end
		BecomeBeardling(target)
	elseif (target:HasTag("manrabbit") and target.components.health.currenthealth <= TUNING.BUNNYMAN_HEALTH / 2) then
		if target.checktask then target.checktask:Cancel() end
		SetBeardlord(target)
	end
	
	return true
end

local function AuraCorruptFlowers(inst, range)
    local hitcount = 0
	local pt = Vector3(inst.Transform:GetWorldPosition() )
	local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, range)
	for i,ent in ipairs(ents) do
		if (ent.prefab == "flower") then
			ent.components.health.currenthealth = ent.components.health.currenthealth - 1
			if (ent.components.health.currenthealth <= 1) then
				BecomeCorruptFlower(ent)
			end
		end
	end
    return hitcount
end

local function updateAura(inst, dt)
	auraRange = dementorAuraMaxRange - (inst.components.sanity.current / dementorMaxSanity) * (dementorAuraMaxRange - 3)
	inst.Light:SetRadius(auraRange)
	AuraCorruptFlowers(inst, auraRange)
	local hits = inst.components.combat:DoAreaAttack(inst, auraRange, inst, validateTarget)
	inst.components.sanity:DoDelta(TUNING.SANITY_TINY / 10 * hits)
end

local fn = function(inst)
	-- choose which sounds this character will play
	inst.soundsname = "wilton"

	-- a minimap icon must be specified
	inst.MiniMapEntity:SetIcon( "wilton.png" )
	
	-- Dementor don't have the usual friends
	inst:AddTag("monster")
	inst:AddTag("ghost")
	
	-- Dementor stats
	inst.components.health.maxhealth = dementorMaxHealth
	inst.components.sanity.max = dementorMaxSanity
	inst.components.sanity.dapperness = TUNING.CRAZINESS_SMALL
	inst.components.temperature.hurtrate = 8
	
	-- Dementors don't get hungry
	inst.components.hunger:Pause()
	inst.components.hunger:SetRate(0)
	inst.components.hunger:SetKillRate(0)
	
	-- Demetor have some affinity to magic
	inst.components.builder.magic_bonus = 2
	
	-- Damaging aura --
	inst:AddComponent("weapon")
	inst.components.weapon:SetDamage(TUNING.SPEAR_DAMAGE * 0.1)
	-- Emit light at day to show the soulstealing range
	inst.entity:AddLight()
	inst.Light:SetRadius(dementorAuraMaxRange)
    inst.Light:SetFalloff(0.99)
    inst.Light:SetIntensity(0.99)
	inst.Light:SetColour(5/255,255/255,5/255)
	inst.Light:Enable(true)
	inst:ListenForEvent("daytime", function() switchAuraDisplay(inst) end , GetWorld())
	inst:ListenForEvent("nighttime", function() switchAuraDisplay(inst) end , GetWorld())
	inst.updateTask = inst:DoPeriodicTask(1, updateAura, nil, 1)
	inst.AnimState:SetBuild("demen") -- Manually changed in the build.bin file
end

STRINGS.CHARACTER_TITLES.dementor = "Dementor"
STRINGS.CHARACTER_NAMES.dementor = "Dementor"
STRINGS.CHARACTER_DESCRIPTIONS.dementor = " *Feeds on fright\n *Does not starve\n *Is a monster"
STRINGS.CHARACTER_QUOTES.dementor = "\"Whooosh...\""
STRINGS.CHARACTERS.DEMENTOR = {}
STRINGS.CHARACTERS.DEMENTOR.DESCRIBE = {}
STRINGS.CHARACTERS.DEMENTOR.DESCRIBE.GENERIC = "Whooosh..."
	
return MakePlayerCharacter("dementor", prefabs, assets, fn, start_inv)


